@@ -34,6 +34,7 @@ import java.util.List;
 import java.util.concurrent.atomic.AtomicReference;
 
 import org.apache.hc.client5.http.AuthenticationStrategy;
+import org.apache.hc.client5.http.EndpointInfo;
 import org.apache.hc.client5.http.HttpRoute;
 import org.apache.hc.client5.http.RouteTracker;
 import org.apache.hc.client5.http.SchemePortResolver;
@@ -44,7 +45,6 @@ import org.apache.hc.client5.http.async.AsyncExecRuntime;
 import org.apache.hc.client5.http.auth.AuthExchange;
 import org.apache.hc.client5.http.auth.ChallengeType;
 import org.apache.hc.client5.http.config.RequestConfig;
-import org.apache.hc.client5.http.impl.TunnelRefusedException;
 import org.apache.hc.client5.http.impl.auth.AuthCacheKeeper;
 import org.apache.hc.client5.http.impl.auth.HttpAuthenticator;
 import org.apache.hc.client5.http.impl.routing.BasicRouteDirector;
@@ -73,7 +73,6 @@ import org.apache.hc.core5.http.nio.CapacityChannel;
 import org.apache.hc.core5.http.nio.DataStreamChannel;
 import org.apache.hc.core5.http.nio.RequestChannel;
 import org.apache.hc.core5.http.protocol.HttpContext;
-import org.apache.hc.core5.http.protocol.HttpCoreContext;
 import org.apache.hc.core5.http.protocol.HttpProcessor;
 import org.apache.hc.core5.util.Args;
 import org.slf4j.Logger;
@@ -121,6 +120,7 @@ public final class AsyncConnectExec implements AsyncExecChainHandler {
         final RouteTracker tracker;
 
         volatile boolean challenged;
+        volatile HttpResponse response;
         volatile boolean tunnelRefused;
 
     }
@@ -176,11 +176,7 @@ public final class AsyncConnectExec implements AsyncExecChainHandler {
                     }));
         } else {
             if (execRuntime.isEndpointConnected()) {
-                try {
-                    chain.proceed(request, entityProducer, scope, asyncExecCallback);
-                } catch (final HttpException | IOException ex) {
-                    asyncExecCallback.failed(ex);
-                }
+                proceedConnected(request, entityProducer, scope, chain, asyncExecCallback);
             } else {
                 proceedToNextHop(state, request, entityProducer, scope, chain, asyncExecCallback);
             }
@@ -257,68 +253,61 @@ public final class AsyncConnectExec implements AsyncExecChainHandler {
                 }));
                 break;
 
-                case HttpRouteDirector.TUNNEL_TARGET:
-                    try {
-                        final HttpHost proxy = route.getProxyHost();
-                        final HttpHost target = route.getTargetHost();
-                        if (LOG.isDebugEnabled()) {
-                            LOG.debug("{} create tunnel", exchangeId);
-                        }
-                        createTunnel(state, proxy, target, scope, new AsyncExecCallback() {
-
-                            @Override
-                            public AsyncDataConsumer handleResponse(
-                                    final HttpResponse response,
-                                    final EntityDetails entityDetails) throws HttpException, IOException {
-                                return asyncExecCallback.handleResponse(response, entityDetails);
-                            }
-
-                            @Override
-                            public void handleInformationResponse(
-                                    final HttpResponse response) throws HttpException, IOException {
-                                asyncExecCallback.handleInformationResponse(response);
-                            }
-
-                            @Override
-                            public void completed() {
-                                if (!execRuntime.isEndpointConnected()) {
-                                    // Remote endpoint disconnected. Need to start over
-                                    if (LOG.isDebugEnabled()) {
-                                        LOG.debug("{} proxy disconnected", exchangeId);
-                                    }
-                                    state.tracker.reset();
-                                }
-                                if (state.challenged) {
-                                    if (LOG.isDebugEnabled()) {
-                                        LOG.debug("{} proxy authentication required", exchangeId);
-                                    }
-                                    proceedToNextHop(state, request, entityProducer, scope, chain, asyncExecCallback);
-                                } else {
-                                    if (state.tunnelRefused) {
-                                        if (LOG.isDebugEnabled()) {
-                                            LOG.debug("{} tunnel refused", exchangeId);
-                                        }
-                                        asyncExecCallback.failed(new TunnelRefusedException("Tunnel refused", null));
-                                    } else {
-                                        if (LOG.isDebugEnabled()) {
-                                            LOG.debug("{} tunnel to target created", exchangeId);
-                                        }
-                                        tracker.tunnelTarget(false);
-                                        proceedToNextHop(state, request, entityProducer, scope, chain, asyncExecCallback);
-                                    }
-                                }
-                            }
-
-                        @Override
-                        public void failed(final Exception cause) {
-                            execRuntime.markConnectionNonReusable();
-                            asyncExecCallback.failed(cause);
-                        }
-
-                    });
-                } catch (final HttpException | IOException ex) {
-                    asyncExecCallback.failed(ex);
+            case HttpRouteDirector.TUNNEL_TARGET:
+                final HttpHost proxy = route.getProxyHost();
+                final HttpHost target = route.getTargetHost();
+                if (LOG.isDebugEnabled()) {
+                    LOG.debug("{} create tunnel", exchangeId);
                 }
+                createTunnel(state, proxy, target, scope, new AsyncExecCallback() {
+
+                    @Override
+                    public AsyncDataConsumer handleResponse(final HttpResponse response, final EntityDetails entityDetails) throws HttpException, IOException {
+                        return asyncExecCallback.handleResponse(response, entityDetails);
+                    }
+
+                    @Override
+                    public void handleInformationResponse(final HttpResponse response) throws HttpException, IOException {
+                        asyncExecCallback.handleInformationResponse(response);
+                    }
+
+                    @Override
+                    public void completed() {
+                        if (!execRuntime.isEndpointConnected()) {
+                            // Remote endpoint disconnected. Need to start over
+                            if (LOG.isDebugEnabled()) {
+                                LOG.debug("{} proxy disconnected", exchangeId);
+                            }
+                            state.tracker.reset();
+                        }
+                        if (state.challenged) {
+                            if (LOG.isDebugEnabled()) {
+                                LOG.debug("{} proxy authentication required", exchangeId);
+                            }
+                            proceedToNextHop(state, request, entityProducer, scope, chain, asyncExecCallback);
+                        } else {
+                            if (state.tunnelRefused) {
+                                if (LOG.isDebugEnabled()) {
+                                    LOG.debug("{} tunnel refused", exchangeId);
+                                }
+                                asyncExecCallback.completed();
+                            } else {
+                                if (LOG.isDebugEnabled()) {
+                                    LOG.debug("{} tunnel to target created", exchangeId);
+                                }
+                                tracker.tunnelTarget(false);
+                                proceedToNextHop(state, request, entityProducer, scope, chain, asyncExecCallback);
+                            }
+                        }
+                    }
+
+                    @Override
+                    public void failed(final Exception cause) {
+                        execRuntime.markConnectionNonReusable();
+                        asyncExecCallback.failed(cause);
+                    }
+
+                });
                 break;
 
             case HttpRouteDirector.TUNNEL_PROXY:
@@ -363,11 +352,7 @@ public final class AsyncConnectExec implements AsyncExecChainHandler {
                 if (LOG.isDebugEnabled()) {
                     LOG.debug("{} route fully established", exchangeId);
                 }
-                try {
-                    chain.proceed(request, entityProducer, scope, asyncExecCallback);
-                } catch (final HttpException | IOException ex) {
-                    asyncExecCallback.failed(ex);
-                }
+                proceedConnected(request, entityProducer, scope, chain, asyncExecCallback);
                 break;
 
             default:
@@ -380,7 +365,7 @@ public final class AsyncConnectExec implements AsyncExecChainHandler {
             final HttpHost proxy,
             final HttpHost nextHop,
             final AsyncExecChain.Scope scope,
-            final AsyncExecCallback asyncExecCallback) throws HttpException, IOException {
+            final AsyncExecCallback asyncExecCallback) {
 
         final CancellableDependency operation = scope.cancellableDependency;
         final HttpClientContext clientContext = scope.clientContext;
@@ -449,7 +434,7 @@ public final class AsyncConnectExec implements AsyncExecChainHandler {
             public void consumeResponse(final HttpResponse response,
                                         final EntityDetails entityDetails,
                                         final HttpContext httpContext) throws HttpException, IOException {
-                clientContext.setAttribute(HttpCoreContext.HTTP_RESPONSE, response);
+                clientContext.setResponse(response);
                 proxyHttpProcessor.process(response, entityDetails, clientContext);
 
                 final int status = response.getCode();
@@ -517,7 +502,7 @@ public final class AsyncConnectExec implements AsyncExecChainHandler {
             final HttpHost proxy,
             final HttpResponse response,
             final HttpClientContext context) {
-        final RequestConfig config = context.getRequestConfig();
+        final RequestConfig config = context.getRequestConfigOrDefault();
         if (config.isAuthenticationEnabled()) {
             final boolean proxyAuthRequested = authenticator.isChallenged(proxy, ChallengeType.PROXY, response, proxyAuthExchange, context);
 
@@ -543,4 +528,24 @@ public final class AsyncConnectExec implements AsyncExecChainHandler {
         return false;
     }
 
+    private void proceedConnected(
+            final HttpRequest request,
+            final AsyncEntityProducer entityProducer,
+            final AsyncExecChain.Scope scope,
+            final AsyncExecChain chain,
+            final AsyncExecCallback asyncExecCallback) {
+        final AsyncExecRuntime execRuntime = scope.execRuntime;
+        final HttpClientContext clientContext = scope.clientContext;
+        final EndpointInfo endpointInfo = execRuntime.getEndpointInfo();
+        if (endpointInfo != null) {
+            clientContext.setProtocolVersion(endpointInfo.getProtocol());
+            clientContext.setSSLSession(endpointInfo.getSslSession());
+        }
+        try {
+            chain.proceed(request, entityProducer, scope, asyncExecCallback);
+        } catch (final HttpException | IOException ex) {
+            asyncExecCallback.failed(ex);
+        }
+    }
+
 }
