@@ -33,6 +33,7 @@ import org.apache.hc.client5.http.HttpRoute;
 import org.apache.hc.client5.http.async.AsyncExecCallback;
 import org.apache.hc.client5.http.async.AsyncExecChain;
 import org.apache.hc.client5.http.async.AsyncExecChainHandler;
+import org.apache.hc.client5.http.impl.ChainElement;
 import org.apache.hc.client5.http.protocol.HttpClientContext;
 import org.apache.hc.core5.annotation.Contract;
 import org.apache.hc.core5.annotation.Internal;
@@ -43,7 +44,7 @@ import org.apache.hc.core5.http.HttpRequest;
 import org.apache.hc.core5.http.HttpResponse;
 import org.apache.hc.core5.http.nio.AsyncDataConsumer;
 import org.apache.hc.core5.http.nio.AsyncEntityProducer;
-import org.apache.hc.core5.http.nio.entity.NoopEntityConsumer;
+import org.apache.hc.core5.http.nio.entity.DiscardingEntityConsumer;
 import org.apache.hc.core5.http.support.BasicRequestBuilder;
 import org.apache.hc.core5.util.Args;
 import org.apache.hc.core5.util.TimeValue;
@@ -61,6 +62,22 @@ import org.slf4j.LoggerFactory;
  * endpoint is delegated to the next executor in the request execution
  * chain.
  * </p>
+ * <p>
+ * If this handler is active, pay particular attention to the placement
+ * of other handlers within the handler chain relative to the retry handler.
+ * Use {@link ChainElement#RETRY} as name when referring to this handler.
+ * </p>
+ * <p>
+ * If a custom handler is placed <b>before</b> the retry handler, the handler will
+ * see the initial request and the final outcome after the last retry. Elapsed time
+ * will account for any delays imposed by the retry handler.
+ * </p>
+ *
+ * <p>
+ * A custom handler which is placed <b>after</b> the retry handler will be invoked for
+ * each individual retry. Elapsed time will measure each individual http request,
+ * without the delay imposed by the retry handler.
+ * </p>
  *
  * @since 5.0
  */
@@ -113,10 +130,9 @@ public final class AsyncHttpRequestRetryExec implements AsyncExecChainHandler {
                     if (LOG.isDebugEnabled()) {
                         LOG.debug("{} retrying request in {}", exchangeId, state.delay);
                     }
-                    return new NoopEntityConsumer();
-                } else {
-                    return asyncExecCallback.handleResponse(response, entityDetails);
+                    return new DiscardingEntityConsumer<>();
                 }
+                return asyncExecCallback.handleResponse(response, entityDetails);
             }
 
             @Override
@@ -131,7 +147,13 @@ public final class AsyncHttpRequestRetryExec implements AsyncExecChainHandler {
                     if (entityProducer != null) {
                        entityProducer.releaseResources();
                     }
-                    scope.scheduler.scheduleExecution(request, entityProducer, scope, asyncExecCallback, state.delay);
+                    scope.scheduler.scheduleExecution(
+                            request,
+                            entityProducer,
+                            scope,
+                            (r, e, s, c) -> execute(r, e, s, chain, c),
+                            asyncExecCallback,
+                            state.delay);
                 } else {
                     asyncExecCallback.completed();
                 }
@@ -159,8 +181,15 @@ public final class AsyncHttpRequestRetryExec implements AsyncExecChainHandler {
                             entityProducer.releaseResources();
                         }
                         state.retrying = true;
-                        scope.execCount.incrementAndGet();
-                        scope.scheduler.scheduleExecution(request, entityProducer, scope, asyncExecCallback, state.delay);
+                        final int execCount = scope.execCount.incrementAndGet();
+                        state.delay = retryStrategy.getRetryInterval(request, (IOException) cause, execCount - 1, clientContext);
+                        scope.scheduler.scheduleExecution(
+                                request,
+                                entityProducer,
+                                scope,
+                                (r, e, s, c) -> execute(r, e, s, chain, c),
+                                asyncExecCallback,
+                                state.delay);
                         return;
                     }
                 }
