@@ -65,6 +65,7 @@ import org.apache.hc.core5.http.NameValuePair;
 import org.apache.hc.core5.http.message.BasicHeaderValueFormatter;
 import org.apache.hc.core5.http.message.BasicNameValuePair;
 import org.apache.hc.core5.http.protocol.HttpContext;
+import org.apache.hc.core5.net.PercentCodec;
 import org.apache.hc.core5.util.Args;
 import org.apache.hc.core5.util.CharArrayBuffer;
 import org.slf4j.Logger;
@@ -113,6 +114,24 @@ public class DigestScheme implements AuthScheme, Serializable {
     private boolean complete;
     private transient ByteArrayBuilder buffer;
 
+    /**
+     * Flag indicating whether username hashing is supported.
+     * <p>
+     * This flag is used to determine if the server supports hashing of the username
+     * as part of the Digest Access Authentication process. When set to {@code true},
+     * the client is expected to hash the username using the same algorithm used for
+     * hashing the credentials. This is in accordance with Section 3.4.4 of RFC 7616.
+     * </p>
+     * <p>
+     * The default value is {@code false}, indicating that username hashing is not
+     * supported. If the server requires username hashing (indicated by the
+     * {@code userhash} parameter in the  a header set to {@code true}),
+     * this flag should be set to {@code true} to comply with the server's requirements.
+     * </p>
+     */
+    private boolean userhashSupported = false;
+
+
     private String lastNonce;
     private long nounceCount;
     private String cnonce;
@@ -122,15 +141,22 @@ public class DigestScheme implements AuthScheme, Serializable {
     private UsernamePasswordCredentials credentials;
 
     public DigestScheme() {
-        this(StandardCharsets.ISO_8859_1);
-    }
-
-    public DigestScheme(final Charset charset) {
-        this.defaultCharset = charset != null ? charset : StandardCharsets.ISO_8859_1;
+        this.defaultCharset =  StandardCharsets.UTF_8;
         this.paramMap = new HashMap<>();
         this.complete = false;
     }
 
+    /**
+     * @deprecated This constructor is deprecated to enforce the use of {@link StandardCharsets#UTF_8} encoding
+     * in compliance with RFC 7616 for HTTP Digest Access Authentication. Use the default constructor {@link #DigestScheme()} instead.
+     *
+     * @param charset the {@link Charset} set to be used for encoding credentials. This parameter is ignored as UTF-8 is always used.
+     */
+    @Deprecated
+    public DigestScheme(final Charset charset) {
+        this();
+    }
+
     public void initPreemptive(final Credentials credentials, final String cnonce, final String realm) {
         Args.notNull(credentials, "Credentials");
         Args.check(credentials instanceof UsernamePasswordCredentials,
@@ -170,6 +196,10 @@ public class DigestScheme implements AuthScheme, Serializable {
         if (this.paramMap.isEmpty()) {
             throw new MalformedChallengeException("Missing digest auth parameters");
         }
+
+        final String userHashValue = this.paramMap.get("userhash");
+        this.userhashSupported = "true".equalsIgnoreCase(userHashValue);
+
         this.complete = true;
     }
 
@@ -197,7 +227,7 @@ public class DigestScheme implements AuthScheme, Serializable {
         }
 
         if (LOG.isDebugEnabled()) {
-            final HttpClientContext clientContext = HttpClientContext.adapt(context);
+            final HttpClientContext clientContext = HttpClientContext.cast(context);
             final String exchangeId = clientContext.getExchangeId();
             LOG.debug("{} No credentials found for auth scope [{}]", exchangeId, authScope);
         }
@@ -314,6 +344,28 @@ public class DigestScheme implements AuthScheme, Serializable {
 
         a1 = null;
         a2 = null;
+
+
+        // Extract username and username*
+        String username = credentials.getUserName();
+        String encodedUsername = null;
+        // Check if 'username' has invalid characters and use 'username*'
+        if (username != null && containsInvalidABNFChars(username)) {
+            encodedUsername = "UTF-8''" + PercentCodec.RFC5987.encode(username);
+        }
+
+        final String usernameForDigest;
+        if (this.userhashSupported) {
+            final String usernameRealm = username + ":" + realm;
+            final byte[] hashedBytes = digester.digest(usernameRealm.getBytes(StandardCharsets.UTF_8));
+            usernameForDigest = formatHex(hashedBytes); // Use hashed username for digest
+            username = usernameForDigest;
+        } else if (encodedUsername != null) {
+            usernameForDigest = encodedUsername; // Use encoded username for digest
+        } else {
+            usernameForDigest = username; // Use regular username for digest
+        }
+
         // 3.2.2.2: Calculating digest
         if ("MD5-sess".equalsIgnoreCase(algorithm)) {
             // H( unq(username-value) ":" unq(realm-value) ":" passwd )
@@ -321,13 +373,13 @@ public class DigestScheme implements AuthScheme, Serializable {
             //      ":" unq(cnonce-value)
 
             // calculated one per session
-            buffer.append(credentials.getUserName()).append(":").append(realm).append(":").append(credentials.getUserPassword());
+            buffer.append(username).append(":").append(realm).append(":").append(credentials.getUserPassword());
             final String checksum = formatHex(digester.digest(this.buffer.toByteArray()));
             buffer.reset();
             buffer.append(checksum).append(":").append(nonce).append(":").append(cnonce);
         } else {
             // unq(username-value) ":" unq(realm-value) ":" passwd
-            buffer.append(credentials.getUserName()).append(":").append(realm).append(":").append(credentials.getUserPassword());
+            buffer.append(username).append(":").append(realm).append(":").append(credentials.getUserPassword());
         }
         a1 = buffer.toByteArray();
 
@@ -388,7 +440,17 @@ public class DigestScheme implements AuthScheme, Serializable {
         buffer.append(StandardAuthScheme.DIGEST + " ");
 
         final List<BasicNameValuePair> params = new ArrayList<>(20);
-        params.add(new BasicNameValuePair("username", credentials.getUserName()));
+        if (this.userhashSupported) {
+            // Use hashed username for the 'username' parameter
+            params.add(new BasicNameValuePair("username", usernameForDigest));
+            params.add(new BasicNameValuePair("userhash", "true"));
+        } else if (encodedUsername != null) {
+            // Use encoded 'username*' parameter
+            params.add(new BasicNameValuePair("username*", encodedUsername));
+        } else {
+            // Use regular 'username' parameter
+            params.add(new BasicNameValuePair("username", username));
+        }
         params.add(new BasicNameValuePair("realm", realm));
         params.add(new BasicNameValuePair("nonce", nonce));
         params.add(new BasicNameValuePair("uri", uri));
@@ -488,4 +550,43 @@ public class DigestScheme implements AuthScheme, Serializable {
         return getName() + this.paramMap;
     }
 
+    /**
+     * Checks if a given string contains characters that are not allowed
+     * in an ABNF quoted-string as per standard specifications.
+     * <p>
+     * The method checks for:
+     * - Control characters (ASCII 0x00 to 0x1F and 0x7F).
+     * - Characters outside the printable ASCII range (above 0x7E).
+     * - Double quotes (&quot;) and backslashes (\), which are not allowed.
+     * </p>
+     *
+     * @param value The string to be checked for invalid ABNF characters.
+     * @return {@code true} if invalid characters are found, {@code false} otherwise.
+     * @throws IllegalArgumentException if the input string is null.
+     */
+    private boolean containsInvalidABNFChars(final String value) {
+        if (value == null) {
+            throw new IllegalArgumentException("Input string should not be null.");
+        }
+
+        for (int i = 0; i < value.length(); i++) {
+            final char c = value.charAt(i);
+
+            // Check for control characters and DEL
+            if (c <= 0x1F || c == 0x7F) {
+                return true;
+            }
+
+            // Check for characters outside the range 0x20 to 0x7E
+            if (c > 0x7E) {
+                return true;
+            }
+
+            // Exclude double quotes and backslash
+            if (c == '"' || c == '\\') {
+                return true;
+            }
+        }
+        return false;
+    }
 }
